#include "fwl5.h"
#include "dialog.h"

//----------------------------------------------------------------------------
int
  Dialog::RunModal
    ( BaseWin* parent
    , int resID
    )
{
  modal = TRUE;
  return DialogBoxParam( Application::FWGetAppInstance()
                , MAKEINTRESOURCE(resID)
                , parent?parent->GetHWND():0
                , Dialog::Handler
                , (LONG)this
                );
}

//----------------------------------------------------------------------------
BOOL Dialog::Create()
{
  modal = FALSE;
  HWND h = CreateDialogParam( Application::FWGetAppInstance()
                , MAKEINTRESOURCE(rsrc)
                , 0
                , Dialog::Handler
                , (LONG)this
                );
  SetHWND(h);
  if (h)
  {
    GetWindowRect(h, &outerRect);
    style = GetWindowLong(h, GWL_STYLE);
    return TRUE;
  }
  return FALSE;
}

//----------------------------------------------------------------------------
int
  Dialog::RunModeless
    ( BaseWin* parent
    , int resID
    )
{
  if (GetHWND() == 0)
  {
    modal = FALSE;
    HWND h = CreateDialogParam( Application::FWGetAppInstance()
                  , MAKEINTRESOURCE(resID)
                  , parent?parent->GetHWND():0
                  , Dialog::Handler
                  , (LONG)this
                  );
    SetHWND(h);
    if (h)
    {
      GetWindowRect(h, &outerRect);
      style = GetWindowLong(h, GWL_STYLE);
    }
  }
  return (GetHWND()!=0);
}

//----------------------------------------------------------------------------
BOOL CALLBACK Dialog::Handler(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	Dialog* t = (Dialog*)GetWindowLong(hdlg, DWL_USER);
	BOOL rv = FALSE; // I didn't handle it
	int presDoing;
	switch(msg)
	{
	case WM_INITDIALOG:
		SetWindowLong(hdlg, DWL_USER, lParam);
		rv = TRUE; // set focus for me
      ((Dialog*)lParam)->doingProc = TRUE;
		((Dialog*)lParam)->SetHWND(hdlg);
		((Dialog*)lParam)->OnCreate();
		((Dialog*)lParam)->doingProc = FALSE;
		break;

	case WM_COMMAND:
		presDoing = t->doingProc;
		t->doingProc = TRUE;
		t->Execute(LOWORD(wParam), HIWORD(wParam));
		t->doingProc = presDoing;
		rv = TRUE; // I did handle it.
		break;

	case WM_DESTROY:
    t->OnDestroy();
		t->SetHWND(0);
		break;

   case WM_PAINT:
		HDC h = BeginPaint(hdlg, &ps);
		t->OnPaint(CDC(h), ps.rcPaint);
		EndPaint(hdlg, &ps);
      break;

#if 0
	case WM_CLOSE:
		//MessageBox(hdlg, "close attempt", "handler", MB_OK);
		if (t->QueryClose())
		{
			//DestroyWindow(hdlg);
			EndDialog(hdlg, 0);
		}
		rv = TRUE;
		break;
#endif
	}

	return rv;
}

//----------------------------------------------------------------------------
Dialog::Dialog() : BaseWin(), rsrc(0)
{
  doingProc = FALSE;

}

//----------------------------------------------------------------------------
Dialog::Dialog(int resID) : BaseWin(), rsrc(resID)
{
  doingProc = FALSE;
}

//----------------------------------------------------------------------------
Dialog::~Dialog()
{
}

//----------------------------------------------------------------------------
void Dialog::Terminate(int retval)
{
  if (modal && doingProc)
  {
    EndDialog(GetHWND(), retval);
  }
  else
  {
    assert(modal);
    assert(doingProc);
    assert(!"Can't terminate??");
  }
}


void Dialog::SelfCenter()
{
  RECT wRect;
  GetWindowRect(GetHWND(), &wRect);
  RECT pRect;
  HWND par = GetParent(GetHWND());
  GetWindowRect(par, &pRect);
  int width = wRect.right-wRect.left;
  int height=wRect.bottom-wRect.top;
  int nLeft = pRect.left + ((pRect.right-pRect.left)-width)/2;
  int nTop = pRect.top + ((pRect.bottom-pRect.top)-height)/2;
  MoveWindow(GetHWND(), nLeft, nTop, width, height, FALSE);
}
