#ifndef _LINKER
#define _LINKER

/************************************************************************\
						Dynamic Link List Macros
\************************************************************************/
// Free dynamic memory through entire link list & remove all list entries
//#define KILL_DELETE_LINK(tlink)	{ while( (tlink).ptrabs(0) ) { delete (tlink).ptrabs(0); (tlink).removeabs(0); } }
#define KILL_DELETE_LINK(tlink)	(tlink).Flush()

// Remove all list entries
//#define KILL_LINK(tlink) { while( (tlink).ptrabs(0) ) (tlink).removeabs(0); }
#define KILL_LINK(tlink) (tlink).Flush()

/************************************************************************\
						Dynamic Link List Class
\************************************************************************/
class LINKLIST
{
private:
        struct LinkStruct 
        {
                int handle;
                void *data;
                LinkStruct *prev,*next;
        };

        LinkStruct *head,*tail,*current;
        int currentindex,sizecount,nexthandle;
public:
        LINKLIST() 
        { 
           currentindex=-1; head=tail=current=NULL; 
           sizecount=nexthandle=0;
        }
        ~LINKLIST() { while ( !removeabs(0) ); }

        int removehead();
        void* gethead();

        int add(void *ptr);
        int count(void) { return sizecount; }
        void *getabs(int id);
        void *operator[] (int id) { return getabs(id); }

        int removeabs(int id);
        int removeptr(void* ptr);
        // *** These are the only function which utilize "handles" ***
        int remove(int handle); 
        void *get(int handle);
};

template <class T> class TLINK
{
	LINKLIST	list;
public:
	TLINK() { }
	~TLINK() 
   { 
        //~LINKLIST() { while ( !removeabs(0) ); }
        // Left to itself, the nodes would be deleted,
        // but the items would remain.
     Flush();
   }
   void Flush()
   {
     T* thing;
     while (0!=(thing = static_cast<T*>(list.gethead())))
     {
       delete thing;
       list.removehead();
     }
   }

	int		count(void)	{ return list.count(); }

	int		removeabs(int id) { return list.removeabs(id); }
	int		remove(int handle) { return list.remove(handle); }
   
   int removeptr(T* ptr)
   {
     int r=list.removeptr(ptr);
     if (r>0) delete ptr;
     return r;
   }

    T&		operator[] (int id) { return *((T *)list.getabs(id)); }

	T&		itemabs(int id) { return *((T *)list.getabs(id)); }
	T&		item(int id) { return *((T *)list.get(id)); }

	T*		ptrabs(int id) { return ((T *)list.getabs(id)); }
	T*		ptr(int id) { return ((T *)list.get(id)); }

	int		addptr(T *ptr) { return list.add(ptr); }
//	int		additem(T& ptr)
//	{
//   People on ludes should not copy.
//		T *n = new T;
//		*n = ptr;
//		return addptr(n);
//	}
private:
  // should not happen (causes holy hell)
  // In other words, if copies are allowed, we have to make
  // two copies of the list, and things evidently belong to
  // two lists instead of one.
  // None of the clients of this class are sophisticated
  // enough classes themselves to deal with the ramifications.
  TLINK(const TLINK<T>& other)
  {
  }
  // should not happen (causes holy hell)
  TLINK& operator=(const TLINK<T>& other)
  {
    return *this;
  }
};

#endif
