#include "fwl5.h"
#include "basewin.h"
//#include "combobox.h"
//#include "palette.h"

//----------------------------------------------------------------------------
BaseWin::~BaseWin()
{
  //assert(!IsWindow(hwnd));
  if (hwnd != 0)
  {
    DestroyWindow(hwnd);
  }
}
//-----------------------------------------------------------------------------
void BaseWin::Destroy()
{
  DestroyWindow(hwnd);
  hwnd = 0;
}
//----------------------------------------------------------------------------
void BaseWin::SetHWND(HWND h)
{
  hwnd = h;
}
//----------------------------------------------------------------------------
HWND BaseWin::GetHWND() const
{
  return hwnd;
}
//----------------------------------------------------------------------------
void BaseWin::UpdateArea
  (const CRect& update)
{
  InvalidateArea(update);
  ::UpdateWindow(hwnd);
}
//----------------------------------------------------------------------------
void BaseWin::UpdateAll()
{
  ::InvalidateRect(hwnd, 0, FALSE);
  ::UpdateWindow(hwnd);
}
//----------------------------------------------------------------------------
void BaseWin::InvalidateArea
  (const CRect& invalid)
{
  InvalidateRect(hwnd, (RECT*)&invalid, FALSE);
}
//----------------------------------------------------------------------------
void BaseWin::InvalidateAll()
{
  ::InvalidateRect(hwnd, 0, FALSE);
}
//----------------------------------------------------------------------------
void BaseWin::Show()
{
  ShowWindow(hwnd, SW_SHOW);
}
//----------------------------------------------------------------------------
void BaseWin::Hide()
{
  ShowWindow(hwnd, SW_HIDE);
}
//----------------------------------------------------------------------------
void BaseWin::Execute
  (int,int)//(int id, int note)
{
}

//----------------------------------------------------------------------------
void BaseWin::SetTitle
  (const char* _title)
{
  title = _title;
  if (hwnd)
  {
    SetWindowText(hwnd, (char*)title.c_str());
  }
}


//----------------------------------------------------------------------------
//Palette BaseWin::GetPalette() const
//{
//  return Palette();
//}


//----------------------------------------------------------------------------
// Set the return value of a dialog box.
// Used by dialog and property sheet classes.
void BaseWin::SetDlgMsgResult(DWORD res)
{
  SetWindowLong(hwnd, DWL_MSGRESULT, res);
}

//----------------------------------------------------------------------------
void BaseWin::SeekAndExecute(CmdAssoc* table, int cmd, int note)
{
  while (table->key != 0)
  {
    if (table->key == cmd)
    {
      (this->*(table->fn))(cmd, note);
      return;
    }
    table++;
  }
}

BOOL BaseWin::Create()
{
  return FALSE;
}

BOOL BaseWin::OnContextMenu(HWND h, const CPoint& where)
{
  return FALSE;
}

LRESULT BaseWin::SendItemMessage(UINT id, UINT msg, WPARAM wParam, LPARAM lParam)
{
  return SendDlgItemMessage(hwnd, id, msg, wParam, lParam);
}

void BaseWin::SetItemCheck(UINT id, BOOL state)
{
  SendItemMessage(id,BM_SETCHECK,state,0);
}

BOOL BaseWin::GetItemCheck(UINT id)
{
  return SendItemMessage(id, BM_GETCHECK,0,0);
}

void BaseWin::EnableItem(UINT id)
{
  EnableWindow(GetDlgItem(hwnd, id), TRUE);
}

void BaseWin::DisableItem(UINT id)
{
  EnableWindow(GetDlgItem(hwnd, id), FALSE);
}

void BaseWin::CheckRadioButton(UINT id1, UINT id2, UINT idcheck)
{
  ::CheckRadioButton(hwnd, id1, id2, idcheck);
}

void BaseWin::SetExStyle(DWORD val)
{
  exStyle = val;
  
  if (hwnd)
  {
    SetWindowLong(hwnd, GWL_EXSTYLE, exStyle);
  }
}

LRESULT BaseWin::SendMessage(UINT msg, WPARAM w, LPARAM l)
{
  return ::SendMessage(hwnd, msg, w, l);
}

string BaseWin::GetItemText(int item, int len)
{
  string r;
  r.resize(len);
  SendDlgItemMessage(hwnd, item, WM_GETTEXT, len, (LPARAM)(LPSTR)r.c_str());
  return r;
}

void BaseWin::SetItemText(int item, const string& text)
{
  SendDlgItemMessage(hwnd, item, WM_SETTEXT, 0, (LPARAM)(LPSTR)text.c_str());
}

string BaseWin::GetTitle()
{
  if (hwnd)
  {
    string r;
    r.resize(128);
    SendMessage(hwnd, WM_GETTEXT, 128, (LPARAM)(LPSTR)r.c_str());
    return r;
  }
  else
  {
    return title;
  }
}
