#ifndef DDBOX
#define DDBOX
#include <ddraw.h>
class DDSurf
{
public:
  class Desc : public DDSURFACEDESC
  {
  public:
    Desc()
    {
      memset((DDSURFACEDESC*)this, 0, sizeof(DDSURFACEDESC));
      dwSize = sizeof(DDSURFACEDESC);
    }
    void SetPrimary()
    {
      dwFlags |= DDSD_CAPS;
      ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
    }
    void SetOffscreenBuffer(int w, int h, int rPlace)
    {
      dwFlags |= DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
      if (rPlace)
        ddsCaps.dwCaps = rPlace;
      else
        ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
      dwWidth = w;
      dwHeight = h;
    }
    void SetFlipping(int nBack)
    {
      dwFlags |= DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
      ddsCaps.dwCaps = DDSCAPS_COMPLEX | DDSCAPS_FLIP | DDSCAPS_PRIMARYSURFACE;
      dwBackBufferCount = nBack;
    }
  };
  void Attach(DDSurf& other)
  {
    hr = p->AddAttachedSurface(other.p);
  }
  void AddDirtyRect(const CRect& r)
  {
    CRect R= r;
    hr = p->AddOverlayDirtyRect(&R);
  }
  void Blt(const DDSurf& src
          , const CRect& d
          , const CRect& s
          , DWORD dwFlags
          )
  {
    CRect D = d;
    CRect S = s;
    hr = p->Blt(&D, src.p, &S, dwFlags, 0);
  }
  void BltFast(const DDSurf& src
              , const CPoint& d
              , const CRect& s
              , DWORD dwTrans
              )
  {
    CRect S = s;
    hr = p->BltFast(d.x, d.y, src.p, &S, dwTrans);
  }
  void DeleteAttached(DDSurf& det)
  {
    hr = p->DeleteAttachedSurface(0, det.p);
  }
  BOOL FillColor(DWORD c)
  {
    DDBLTFX ddbltfx;
    ddbltfx.dwSize = sizeof(ddbltfx);
    ddbltfx.dwFillColor = c;
    hr = p->Blt(0,0,0, DDBLT_COLORFILL|DDBLT_WAIT,&ddbltfx);
    return hr==DD_OK;
  }
  BOOL FillColorRect(DWORD c, const CRect& dest)
  {
    CRect d = dest;
    DDBLTFX ddbltfx;
    ddbltfx.dwSize = sizeof(ddbltfx);
    ddbltfx.dwFillColor = c;
    hr = p->Blt(&d,0,0, DDBLT_COLORFILL|DDBLT_WAIT,&ddbltfx);
    return hr==DD_OK;
  }
  BOOL Flip(DWORD flags)
  {
    hr = p->Flip(0, flags);
    return hr==DD_OK;
  }
  BOOL FlipTo(DDSurf& dest, DWORD flags)
  {
    hr = p->Flip(dest.p, flags);
    return hr==DD_OK;
  }
  DDSurf GetAttachedSurface(DDSCAPS caps)
  {
    DDSurf r;
    IDirectDrawSurface* np=0;
    hr = p->GetAttachedSurface(&caps, &np);
    r.p = np;
    return r;
  }
  HRESULT GetBltStatus(DWORD f)
  {
    return p->GetBltStatus(f);
  }
  DDSCAPS GetCaps()
  {
    DDSCAPS r;
    hr=p->GetCaps(&r);
    return r;
  }
  HDC GetDC()
  {
    HDC r;
    hr = p->GetDC(&r);
    return r;
  }
  HRESULT GetFlipStatus(DWORD f)
  {
    return p->GetFlipStatus(f);
  }
  Desc GetSurfaceDesc()
  {
    Desc r;
    hr = p->GetSurfaceDesc(&r);
    return r;
  }
  BOOL IsLost()
  {
    hr = p->IsLost();
    return hr==DDERR_SURFACELOST;
  }
  HRESULT Lock(Desc &how)
  {
    hr = p->Lock(0, &how, DDLOCK_SURFACEMEMORYPTR, 0);
    return hr;
  }
  HRESULT LockRect(const CRect& where, Desc &how)
  {
    CRect r = where;
    hr = p->Lock(&r, &how, DDLOCK_SURFACEMEMORYPTR, 0);
    return hr;
  }
  HRESULT LockWait(Desc &how)
  {
    hr = p->Lock(0, &how, DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT, 0);
    return hr;
  }
  HRESULT LockRectWait(const CRect& where, Desc &how)
  {
    CRect r = where;
    hr = p->Lock(&r, &how, DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT, 0);
    return hr;
  }
  void ReleaseDC(HDC h)
  {
    hr = p->ReleaseDC(h);
  }
  void Restore()
  {
    hr = p->Restore();
  }
  void Unlock(void* context)
  {
    hr = p->Unlock(context);
  }

 
  DDSurf& operator=(const DDSurf& other)
  {
    p = other.p;
    p->AddRef();
    return *this;
  }
  DDSurf()
  {
    p = 0;
  }
  ~DDSurf()
  {
    p->Release();
  }
private:
  HRESULT hr;
  IDirectDrawSurface* p;
  friend class DDBox;
};
class DDBox
{
public:
  BOOL Create()
  {
    hr = DirectDrawCreate(0, &p, 0);
    return hr==DD_OK;
  }
  BOOL CreateSurface(DDSurf::Desc& desc, DDSurf& surf)
  {
    IDirectDrawSurface* ddsp;
    hr = p->CreateSurface(&desc, &ddsp, 0);
    if (hr==DD_OK)
    {
      surf.p = ddsp;
      return TRUE;
    }
    return FALSE;
  }
  class Caps: public DDCAPS
  {
  public:
    Caps()
    {
      memset((DDCAPS*)this, 0, sizeof(DDCAPS));
      dwSize = sizeof(DDCAPS);
    }
  };
  void GetCaps(Caps& hw, Caps& emu)
  {
    hr = p->GetCaps(&hw, &emu);
  }
  void GetDisplayMode(DDSurf::Desc& desc)
  {
    hr = p->GetDisplayMode(&desc);
  }
  DWORD GetNumFourCCCodes()
  {
    DWORD r=0;
    hr = p->GetFourCCCodes(&r, 0);
    return r;
  }
  void GetFourCCCodes(DWORD c, DWORD* array)
  {
    DWORD r=c;
    hr = p->GetFourCCCodes(&r, array);
  }
  BOOL GetGDISurface(DDSurf& surf)
  {
    IDirectDrawSurface* ddsif;
    hr = p->GetGDISurface(&ddsif);
    if (hr==DD_OK)
    {
      surf.p = ddsif;
      return TRUE;
    }
    return FALSE;
  }
  DWORD GetMonitorFrequence()
  {
    DWORD r=0;
    hr = p->GetMonitorFrequency(&r);
    return r;
  }
  DWORD GetScanLine()
  {
    DWORD r=0;
    hr = p->GetScanLine(&r);
    return r;
  }
  BOOL GetVerticalBlankStatus()
  {
    BOOL r = 0;
    hr = p->GetVerticalBlankStatus(&r);
    return r;
  }
  void Initialize()
  {
    // added for completeness, but harmless and useless
    hr = p->Initialize(0);
  }
  void RestoreDisplayMode()
  {
    hr = p->RestoreDisplayMode();
  }
  void SetCooperativeLevel(HWND h, DWORD flags)
  {
    hr = p->SetCooperativeLevel(h, flags);
  }
  void SetDisplayMode(const CSize& dims, DWORD bits)
  {
    hr = p->SetDisplayMode(dims.cx, dims.cy, bits);
  }
  void WaitForVerticalBlank(DWORD dwFlags)
  {
    // event-using variant not supported.
    hr = p->WaitForVerticalBlank(dwFlags, 0);
  }
  

  void ShowGDI()
  {
    hr = p->FlipToGDISurface();
  }
  DDBox& operator=(const DDBox& other)
  {
    p = other.p;
    p->AddRef();
    return *this;
  }
  DDBox()
  {
    p = 0;
  }
  ~DDBox()
  {
    p->Release();
  }
  HRESULT GetLastResult()
  {
    return hr;
  }
private:
  IDirectDraw* p;
  HRESULT hr; // always changed.
};
#endif
