#include "fwl5.h"
#include "wave2.h"
#include "icstate.h"
#include "navcontr.h"
#include "plstate.h"
#include "gamewin.h"
#include "fsstate.h"
#include <stdio.h>
#include <io.h>
//-----------------------------------------------------------------------------
void InitialCreditState::MouseDown(const CPoint&,int flags)
{
}
int pause=0;
//-----------------------------------------------------------------------------
void InitialCreditState::Key(int code,int)
{
  if (code==VK_PAUSE) pause=!pause;
}
char* initialCredits[] =
{ "ATARI", ""
, "Presents", ""
, "Demo Sequences"
, "from"
, ""
, "the"
, "Jaguar GameFilm"
, ""
, "Caves","of","Fear",""
, "by"
, "D. Schwartz"
, ""
, "Copyright","Atari Corp." //, " Atari Corp."
, ""
, "GameFilm"
, "Patent Pending"
, ""
, "Date","16 March 1996"
, ""
, "City","Montevideo, Uruguay"
, ""
, "Agent","Jack Armstrong"
, ""
//, "is hot on the trail"
//, "of a terrorist "
//, "organisation known"
//, "as the White World"
//, "Order."
, "File","White World Order"
, ""
//, "He is about "
//, "to meet his first "
//, "contact..."
, "The operation begins..."
, ""
, 0
};
// each idle:
// if time expired
//   if page to draw
//     draw a page
//     set timer
//   else
//     end sequence
//
//-----------------------------------------------------------------------------
void FBCPrintf(const FontBits& f, const CPoint c, const CDC& dc, char* format, ...)
{
  static char buf[256];
  va_list         vlist;

  va_start(vlist, format);
  vsprintf(buf, format, vlist);
  va_end( vlist );
  long w=f->MeasureString(buf).cx;
  f->SetCP(CPoint(c.x-w/2, c.y));
  f->DrawString(dc, buf);
}
//-----------------------------------------------------------------------------
char** NextPage(char** p)
{
  while (*p != 0 && **p!=0) p++;
  if (**p==0) p++;
  return p;
}
//-----------------------------------------------------------------------------
void CreditScenePaint
  ( const CDC& dc
  , Window* window
  , char** text
  , FontBits& font
  )
{
  CRect cr = window->GetInnerRect();
  font->SetColor(FONT_WHITE);
  CPoint tpos(160,30);
  int ph = 210;
  if (cr.GetHeight() > 300)
  {
    tpos.x = 320;
    ph = 420;
    font->SetZoom(2);
  }
  int lc=0;
  char** credit = text;
  while (credit[0][0] != 0)
  {
    credit++;
    lc++;
  }
  int dy=font->GetHeight()+2;
  tpos.y = (ph-(dy*lc))/2;

  credit = text;
  while (credit[0][0] != 0)
  {
    char* text = credit[0];
    FBCPrintf(font, tpos, dc, credit[0]);
    tpos.y += dy;
    credit++;
  }
  GdiFlush();
}
static stime=0;
enum CState { RESET, DRAWING, WAITING, FINISHING, FADINGOUT };
static CState state= RESET;
static fadeVol = 0;
//-----------------------------------------------------------------------------
BOOL                                  // TRUE: proceed to next state
  CreditSceneIdle
    ( char**& text                    // current access to text
    , char** credits                  // initialize text to:
    , const char* tune                // name of tune file
    , SoundStream& introMusic         // object to play tune with
    , Window* window                  // window to draw on
    , FontBits& font                  // font to draw with
    , Disks* disks                    // archive info
    )
{
  if (state == RESET)
  {
    stime = 0;
    text = credits;
    char buf[270];
    strcpy(buf, disks->GetCD(0).path.c_str());
    char* cp = strchr(buf, '\\')+1;
    strcpy(cp, tune);
    if (_access(buf,0)==0)
    {
      introMusic.Open(buf);
    }
    else
    {
      strcpy(buf, disks->GetHD().path.c_str());
      char* cp = strchr(buf, '\\')+1;
      strcpy(cp, tune);
      introMusic.Open(buf);
    }
    introMusic.Play(1);
    state = DRAWING;
  }
  if (state == DRAWING)
  {
    stime = GetTickCount();
    state = WAITING;
    CWindowDC dc(window);
    CRect cr = window->GetInnerRect();
    dc.PatBlt(cr, BLACKNESS);
    window->UpdateAll();
    return FALSE;
  }
  else if (state == WAITING && GetTickCount()-stime > 3000 && !pause)
  {
    char** ntext = NextPage(text);
    if (*ntext == 0)
    {
      CWindowDC dc(window);
      CRect cr = window->GetInnerRect();
      dc.PatBlt(cr, BLACKNESS);
      state = FADINGOUT;
      fadeVol = 0;
      text = 0;
    }
    else
    {
      text = ntext;
      state = DRAWING;
    }
    stime=0;
  }
  else if (state == FADINGOUT)
  {
    if (introMusic.ReallyDonePlaying())
    {
      text = 0;
      introMusic.Stop();
      state = RESET;
      return TRUE;
    }
    else if(GetTickCount()-stime > 50)
    {
      stime = GetTickCount();
      fadeVol -= 100;
      if (fadeVol < -3200)
      {
        state = FINISHING;
        fadeVol = -3200;
        introMusic.Stop();
      }
      introMusic.SetVolume(fadeVol);
    }
  }
  else if (state == FINISHING)
  {
    if (introMusic.ReallyDonePlaying())
    {
      text = 0;
      introMusic.Stop();
      state = RESET;
      return TRUE;
    }
  }
  return FALSE;
}
//-----------------------------------------------------------------------------
void InitialCreditState::OnPaint(const CDC& dc, const CRect&)
{
  if (text==0) return;
  Window* window=controller->window;
  FontBits& font=controller->font;
  CreditScenePaint(dc, window, text, font);
}
//-----------------------------------------------------------------------------
void InitialCreditState::Idle()
{
  Window* window=controller->window;
  FontBits& font=controller->font;
  if (CreditSceneIdle( text
                      , initialCredits
                      , "title.wav"
                      , introMusic
                      , window
                      , font
                      , &controller->disks
                      ))
  {
    SetState(new PlayState(controller));
  }
}
//-----------------------------------------------------------------------------
void InitialCreditState::OnAppExit()
{
  introMusic.Stop();
  while (!introMusic.ReallyDonePlaying())
  {
    Sleep(0);
  }
}
//-----------------------------------------------------------------------------
BOOL InitialCreditState::QueryClose()
{
  introMusic.Stop();
  while (!introMusic.ReallyDonePlaying())
  {
    Sleep(0);
  }
  return TRUE;
}
//-----------------------------------------------------------------------------
void FinalCreditState::MouseDown(const CPoint&,int flags)
{
}
//-----------------------------------------------------------------------------
void FinalCreditState::Key(int code,int)
{
  if (code==VK_PAUSE) pause=!pause;
}
char* finalCredits[] =
{ "Jack Armstrong","Neil Sedaka"
, "" //-----------
, "Agent Becker","Neil Armstrong"
, "" //-----------
, "Professor","Louis Armstrong"
, "" //-----------
, "Nurse","Faye Dunaway"
, "" //-----------
, "Stunts"
, "Sean Penn"
, "Alec Baldwin" 
, "Bruce Willis"
, "" //-----------
, "Space Suits", "by", "Botany 500"
, "" //-----------
, "Atari"
, "Exec Producer"
, "Dave Schwartz"
, "" //-----------
, "CetaSoft"
, "Producer"
, "Jack Thornton"
, "" //-----------
, "CetaSoft"
, "Asst Producer"
, "R.J. Knox"
, "" //-----------
, "CetaSoft"
, "Programmers"
, "Mike Enright"
, "Rudy Mathieu"
, "Jamie Fristrom"
, "Jack Thornton"
, "" //-----------
, "NO 3D ARTISTS"
, "WERE EMPLOYED"
, "IN THIS PRODUCTION"
, "" //-----------
, 0
};
//-----------------------------------------------------------------------------
void FinalCreditState::OnPaint(const CDC& dc, const CRect&)
{
  if (text==0) return;
  Window* window=controller->window;
  FontBits& font=controller->font;
  CreditScenePaint(dc, window, text, font);
}
//-----------------------------------------------------------------------------
void FinalCreditState::Idle()
{
  Window* window=controller->window;
  FontBits& font=controller->font;
  if (CreditSceneIdle( text
                      , finalCredits
                      , "credits.wav"    
                      , closingMusic
                      , window
                      , font
                      , &controller->disks
                      ))
  {
    SetState(new FinalStatusState(controller));
  }
}
//-----------------------------------------------------------------------------
void FinalCreditState::OnAppExit()
{
  closingMusic.Stop();
  while (!closingMusic.ReallyDonePlaying())
  {
    Sleep(0);
  }
}
//-----------------------------------------------------------------------------
BOOL FinalCreditState::QueryClose()
{
  closingMusic.Stop();
  while (!closingMusic.ReallyDonePlaying())
  {
    Sleep(0);
  }
  return TRUE;
}
