#include "fwl5.h"
#include <string.h>
#include "proppage.h"
#include "propsh.h"

void CPropertyPage::Setup
  ( const char* tmpl
  , const char* title
  , const char* icon
  )
{
  par = 0;
  memset(&rep,0,sizeof(rep));
  rep.dwSize = sizeof(rep);
  rep.pszTemplate = tmpl;
  rep.hInstance = Application::FWGetAppInstance();
  if (icon)
  {
    rep.dwFlags |= PSP_USEICONID;
    rep.pszIcon = icon;
  }
  if (title) 
  {
    rep.dwFlags |= PSP_USETITLE;
    rep.pszTitle = title;
  }
  rep.pfnDlgProc = handler;
  rep.lParam = (LONG)this;
  //SetHWND((HWND)CreatePropertySheetPage(&rep));
}
CPropertyPage::CPropertyPage
  ( const char* tmpl
  , const char* title
  , const char* icon
  )
{
  Setup(tmpl, title, icon);
}
CPropertyPage::CPropertyPage
  ( const char* tmpl
  , const char* title
  , int icon
  )
{
  Setup(tmpl, title, MAKEINTRESOURCE(icon));
}

BOOL CALLBACK CPropertyPage::handler(HWND h, UINT msg, WPARAM wParam, LPARAM lParam)
{
  NMHDR* nh;
  CPropertyPage* it;
  switch (msg)
  {
  case WM_INITDIALOG:
    it = (CPropertyPage*)((PROPSHEETPAGE*)lParam)->lParam;
    SetWindowLong(h, DWL_USER, (LONG)it);
	it->SetHWND(h);
	it->OnCreate();
    return TRUE;
  case WM_NOTIFY:
    nh = (NMHDR*)lParam;
    it = (CPropertyPage*)GetWindowLong(h, DWL_USER);
    return it->ExecuteNotify(nh->idFrom, nh->hwndFrom, nh->code);
  case WM_COMMAND:
	it = (CPropertyPage*)GetWindowLong(h, DWL_USER);
	it->Execute(LOWORD(wParam),HIWORD(wParam));
	return TRUE;
  }
  return FALSE;
}
int CPropertyPage::ExecuteNotify(int id, HWND from, int code)
{
  switch(code)
  {
  case PSN_APPLY:
    SetDlgMsgResult(OnApply(par));
    return TRUE;
  case PSN_HELP:
    OnHelp(par);
    return TRUE;
  case PSN_KILLACTIVE:
    SetDlgMsgResult(OnKillActive(par));
    return TRUE;
  case PSN_QUERYCANCEL:
    SetDlgMsgResult(OnQueryCancel(par));
    return TRUE;
  case PSN_RESET:
    OnReset(par);
    return TRUE;
  case PSN_SETACTIVE:
	par->SetHWND(from);
    SetDlgMsgResult(OnSetActive(par));
    return TRUE;
  }
  return FALSE;
}
void CPropertyPage::EnableApply() 
{ 
  par->Changed(*this); 
}
void CPropertyPage::DisableApply() 
{ 
  par->Unchanged(*this); 
}
void CPropertyPage::Select()
{
  par->Select(*this);
}