#include "fwl5.h"
#include "pwdialog.h"
#include "dlgitem.h"

BEGIN_COMMAND_TABLE(PWDialog)
  BIND_COMMAND(PWDialog, OnOkay, IDOK)
  BIND_COMMAND(PWDialog, OnCancel, IDCANCEL)
  BIND_COMMAND(PWDialog, OnEdits, IDC_PASSWORD)
  BIND_COMMAND(PWDialog, OnEdits, IDC_REPEATPW)
END_COMMAND_TABLE();

////////////////////////////////////////////////////////////////////////////////
void PWDialog::NoPasswordMessage(const char *reason)
{
  char compositeMsg[384];

  // create the composite message
  wsprintf(compositeMsg, cancelMsg, reason);
  lstrcat(compositeMsg, cancelMsg2);

  // and present message to user
  MessageBeep(MB_ICONEXCLAMATION);
  MessageBox(compositeMsg, cancelTitle, MB_OK | MB_ICONEXCLAMATION);
}
////////////////////////////////////////////////////////////////////////////////
void PWDialog::Initialize()
{
  // clear the password
  password = "";

  // get the handles to our controls
  DialogItem editEntry(this, IDC_PASSWORD);
  DialogItem editRepeat(this, IDC_REPEATPW);

  // clear and enable the first entry box
  editEntry.SetTitle(password);
  editEntry.Enable();

  // clear and disable the second entry box
  editRepeat.SetTitle(password);
  editRepeat.Disable();

  // set the focus on the first box
  editEntry.SetFocus();

  // and tell the user what is expected of him/her
  SetItemText(ID_CAPTION, instructions1);
}
void PWDialog::OnOkay(int cmd, int note)
{
  // look at currently focused control - is it edit 1?
  // get the handles to our controls
  DialogItem editEntry(this, IDC_PASSWORD);
  DialogItem editRepeat(this, IDC_REPEATPW);

  if (GetFocus() == editEntry.GetHWND()) 
  {
    // yep - disable edit one
    editEntry.Disable();

    // and enable the second control (plus ACCEPT button)
    editRepeat.Enable();

    // change the instructions to the user
    SetItemText(ID_CAPTION, instructions2);

    // move on to next control
    PostMessage(GetHWND(), WM_NEXTDLGCTL, 0, 0);
  } 
  else 
  {
    // do both passwords match?
    string pw1 = editEntry.GetTitle();
    string pw2 = editRepeat.GetTitle();
    if (pw1 != pw2)
    {
      // Whoops! Inform user of mismatch
      MessageBeep(MB_ICONEXCLAMATION);
      MessageBox(noMatchMsg, noMatchTitle, MB_OK | MB_ICONEXCLAMATION);
      // re-initialize for retry
      Initialize();
    }
    else 
    {
      // yes, they match: copy password
      password = pw1;

      // if password is "null" (no password), warn user
      if (!password.length())
      {
        NoPasswordMessage(reasonNull);
      }

      // skip out of the dialog
      Terminate(TRUE);
    }
  }
}

void PWDialog::OnCancel(int,int)
{
  MessageBox(noNewPassword, cancelTitle, MB_OK | MB_ICONEXCLAMATION);
  Terminate(FALSE);
}

void PWDialog::OnEdits(int cmd, int note)
{
  if (note == EN_KILLFOCUS)
  {
    if (cmd==IDC_PASSWORD)
    {
      DisableItem(IDC_PASSWORD);
      EnableItem(IDC_REPEATPW);
      DialogItem(this, IDC_REPEATPW).SetFocus();
      // change the instructions to the user
      SetItemText(ID_CAPTION, instructions2);
    }
    else if (cmd==IDC_REPEATPW)
    {
    }
  }
}
