/*********************************************************************\

	RAV Smart Interface
	Written & Designed by Rudy Mathieu

\*********************************************************************/
#include "ravsi.h"

/* Read data into pointer and reports TRUE is not enough data read */
#define READ_DATA(ptr,type)  (sf->Read(ptr,sizeof(type)) < sizeof(type))

RAVSmartInterface::RAVSmartInterface()
{
	memset(&header,0,sizeof(header));
	sampleptr = NULL;
	sf = NULL;
}

BOOL RAVSmartInterface::ReadSample(PFrameSampleStr sample)
{
#define PADDED 1
#if PADDED
   sampleptr = NULL;
   if (READ_DATA(sample,FrameSampleStr))
      return FALSE;

   if (sample->videoSize > header.largestVideoBuffer)
   {
     DebugBreak();
   }
   PBYTE buffer = (PBYTE)sample + sizeof(FrameSampleStr);
   sample->video = buffer; /* Video data pointer */

   DWORD bytes = sf->Read(buffer,sample->videoSize);
   if (bytes == sample->videoSize)
   {
      buffer = buffer + bytes;
      bytes = sf->Read(buffer,sample->audioSize);
      char pad[4];
      unsigned c = (sample->audioSize+sample->videoSize)&3;
      if (c)
      {
         c = 4-c;
	       sf->Read(pad, c);
      }
      if (bytes == sample->audioSize)
      {
        sample->audio = buffer; /* Audio data pointer */
        sampleptr = sample;
        return TRUE; /* We did it!!! */
      }
   }
   return FALSE;
#else // NOT PADDED
   sampleptr = NULL;
   if (READ_DATA(sample,FrameSampleStr))
      return FALSE;

   PBYTE buffer = (PBYTE)sample + sizeof(FrameSampleStr);
   sample->audio = buffer; /* Audio data pointer */

   DWORD bytes = sf->Read(buffer,sample->audioSize);
   if (bytes == sample->audioSize)
   {
      buffer = buffer + bytes;
	  bytes = sf->Read(buffer,sample->videoSize);
	  if (bytes == sample->videoSize)
	  {
	     sample->video = buffer; /* Video data pointer */
	     sampleptr = sample;
		 return TRUE; /* We did it!!! */
	  }
   }
   return FALSE;
#endif
}

BOOL RAVSmartInterface::Open(char *filename, Smartfile *s, void *ptr, long size)
{
	sf = s;
	if (sf->Setup(ptr,size)==FALSE)
		return FALSE;
		
	if (sf->Open(filename)==FALSE)
		return FALSE;

	if (READ_DATA(&header,FileInfoStr))
	   return FALSE;

	return TRUE;
}

BOOL RAVSmartInterface::Close(void)
{
   BOOL rv = TRUE;
   if (sf) 
   {
      rv = sf->Close();
      //sf = NULL;
   }
   return rv;
}

BOOL RAVSmartInterface::Loop(void)
{
   if (sf)
   {
      sf->Seek(sizeof(FileInfoStr));
	  return TRUE;
   }
   return FALSE;
}
