/**********************************************************************\

  Low Level File Handling

\**********************************************************************/

#include "RJMFile.h"
#include "mem.h"

HANDLE Open2Write(const char *filename, int overwrite)
{
	HANDLE h;
	SECURITY_ATTRIBUTES attr={ sizeof(SECURITY_ATTRIBUTES),NULL,FALSE };
    DWORD optFlags = FILE_ATTRIBUTE_NORMAL;
	DWORD cflags;

	if (overwrite)
	{
		cflags = CREATE_ALWAYS;
	} else {
		cflags = OPEN_ALWAYS;
	}

	h = CreateFile(
				filename,GENERIC_WRITE,FILE_SHARE_READ,
				&attr,cflags,optFlags,NULL);
	return h;
}

HANDLE Open2Read(const char *filename,int optFlags)
{
	SECURITY_ATTRIBUTES attr={ sizeof(SECURITY_ATTRIBUTES),NULL,FALSE };

	return CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,
					&attr,OPEN_EXISTING,optFlags| FILE_ATTRIBUTE_NORMAL,NULL);
}

BOOL CloseWinFile(HANDLE fhandle)
{
	return CloseHandle(fhandle);
}

// This function ensures the proper filename extention.
// ext must indicate the dot (.) for example ".ext"
char *changeFileExtention(const char *filename, char *ext)
{
	static char newfilename[255];
	char *ptr;
	strcpy(newfilename,filename);
	ptr=strrchr(newfilename,'.');
	if (ptr==NULL) ptr=newfilename + strlen(newfilename);
	strcpy(ptr,ext);
	return newfilename;
}

void fileFree(void *ptr)
{
	winFree(ptr);
}

PBYTE fileLoad(const char *filename,PDWORD size)
{
	PBYTE buffer=NULL;
	HANDLE fhandle = Open2Read(filename,FILE_ATTRIBUTE_NORMAL);
	if (fhandle)
	{
 		DWORD bytesRead;
		*size = GetFileSize(fhandle,NULL);
		buffer = (PBYTE) winMalloc(*size);
		if (ReadFile(fhandle,buffer,*size,&bytesRead,NULL)==FALSE)
		{
		   winFree(buffer);
		   buffer = NULL;
		}
		CloseHandle(fhandle);
	}
	return buffer;
}
