typedef int bool;
typedef unsigned size_t;
typedef unsigned ptrdiff_t;

class allocator_char {
public:
  typedef size_t    size_type;
  typedef ptrdiff_t difference_type;
  struct types_char {
    typedef char* pointer;
    typedef const char* const_pointer;
    typedef char& reference;
    typedef const char& const_reference;
    typedef char value_type;
  };
//  template <class T> struct types {
//    typedef T*        pointer;
//    typedef const T*  const_pointer;
//    typedef T&        reference;
//    typedef const T&  const_reference;
//    typedef T         value_type;
//  };
  allocator_char();
 ~allocator_char();

//  template<class T> typename types_T::pointer
//    address(types_T::reference x) const;
//  template<class T> typename types_T::const_pointer
//    address(types_T::reference x) const;
//  template<class T, class U> typename types_T::pointer
//    allocate(size_type, types_U::const_pointer hint = 0);
//  template<class T> void deallocate(types_T::pointer p);
//  template<class T>
//    size_type max_size() const;
//  template <class T1, class T2>
//    void construct(T1* p, const T2& val);
//  template <class T>
//    void destroy(T* p);
  types_char::pointer address(types_char::reference x) const;
  types_char::const_pointer address(types_char::const_reference x) const;
  types_char::pointer allocate(size_type, types_char::const_pointer hint = 0);
  void deallocate(types_char::pointer p);
  size_type max_size() const;
  void construct(char* p, const char& val);
  void destroy(char* p);
};
void* operator new(size_t N, allocator_char& a);

int memcmp(const char* s1, const char* s2, size_t n);
char* memchr(const char* s, char a, size_t n);
int strlen(const char* s);
char* memcpy(char* s1, const char* s2, size_t n);
char* memmove(char* s1, const char* s2, size_t n);
char* memset(char* s1, char a, size_t n);

//struct string_char_traits<char> 
struct string_char_traits_char
{
  typedef char char_type;
  static void assign(char& c1, const char& c2) { c1 = c2; }
  static bool eq(const char& c1, const char& c2) { return c1==c2; }
  static bool ne(const char& c1, const char& c2) { return c1!=c2; }
  static bool lt(const char& c1, const char& c2) { return c1<c2; }
  static char eos() { return 0; }
  static int compare(const char* s1, const char* s2, size_t n) { return ::memcmp(s1,s2,n); }
  static const char_type* find(const char* s, int n, const char& a) { return ::memchr(s,a,n); }
  static size_t length(const char* s) { return ::strlen(s); }
  static char* copy(char* s1, const char* s2, size_t n) { return ::memcpy(s1,s2,n); }
  static char* move(char* s1, const char* s2, size_t n) { return ::memmove(s1,s2,n); }
  static char* assign(char* s, size_t n, const char& a) { return ::memset(s,a,n); }
};

//template<class charT, class traits = string_char_traits_char,
//             class Allocator = allocator_char>
template<class charT, class traits,class Allocator>
class basic_string {
  public:
  // types:
    typedef          traits            traits_type;
    typedef /*typename*/ traits::char_type value_type;
    typedef          Allocator         allocator_type;
    typedef /*typename*/ Allocator::size_type       size_type;
    typedef /*typename*/ Allocator::difference_type difference_type;
    typedef /*typename*/ Allocator::types_char::reference       reference;
    typedef /*typename*/ Allocator::types_char::const_reference const_reference;
    typedef /*typename*/ Allocator::types_char::pointer         pointer;
    typedef /*typename*/ Allocator::types_char::const_pointer   const_pointer;
    //typedef implementation_defined                      iterator;
    //typedef implementation_defined                      const_iterator;
    //typedef reverse_iterator<iterator, value_type,
    //                reference, difference_type>               reverse_iterator;
    //typedef reverse_iterator<const_iterator, value_type,
    //                const_reference, difference_type>         const_reverse_iterator;
    typedef char*                      iterator;
    typedef const char*                const_iterator;
    typedef char*                      reverse_iterator;
    typedef const char*                const_reverse_iterator;
    static const size_type npos;// = -1;
    /*explicit*/ basic_string(const Allocator& = Allocator());
    basic_string(const basic_string& str, size_type pos = 0,
                 size_type n = npos, const Allocator& = Allocator());
    basic_string(const charT* s, size_type n, const Allocator& = Allocator());
    basic_string(const charT* s, const Allocator& = Allocator());
    basic_string(size_type n, charT c, const Allocator& = Allocator());
   // template<class InputIterator>
   //   basic_string(InputIterator begin, InputIterator end,
   //                const Allocator& = Allocator());
    basic_string(charT*, charT*);
   ~basic_string();
    basic_string& operator=(const basic_string& str);
    basic_string& operator=(const charT* s);
    basic_string& operator=(charT c);

  // _lib.string.iterators_ iterators:
    iterator       begin();
    const_iterator begin() const;
    iterator       end();
    const_iterator end() const;

    reverse_iterator       rbegin();
    const_reverse_iterator rbegin() const;
    reverse_iterator       rend();
    const_reverse_iterator rend() const;
  // _lib.string.capacity_ capacity:
    size_type size() const;
    size_type length() const;
    size_type max_size() const;
    void resize(size_type n, charT c);
    void resize(size_type n);
    size_type capacity() const;
    void reserve(size_type res_arg);
    bool empty() const;
  // _lib.string.access_ element access:
    charT     operator[](size_type pos) const;
    reference operator[](size_type pos);
    const_reference at(size_type n) const;
    reference       at(size_type n);
  // _lib.string.modifiers_ modifiers:
    basic_string& operator+=(const basic_string& rhs);
    basic_string& operator+=(const charT* s);
    basic_string& operator+=(charT c);
    basic_string& append(const basic_string& str);
    basic_string& append(const basic_string& str, size_type pos,
                         size_type n);
    basic_string& append(const charT* s, size_type n);
    basic_string& append(const charT* s);
    basic_string& append(size_type n, charT c);
    //template<class InputIterator>
    //  basic_string& append(InputIterator first, InputIterator last);
    basic_string& append(charT* first, charT* last);
    basic_string& assign(const basic_string&);
    basic_string& assign(const basic_string& str, size_type pos, size_type n);
    basic_string& assign(const charT* s, size_type n);
    basic_string& assign(const charT* s);
    basic_string& assign(size_type n, charT c);
    //template<class InputIterator>
    //  basic_string& assign(InputIterator first, InputIterator last);
    basic_string& assign(charT* first, charT* last);

    basic_string& insert(size_type pos1, const basic_string& str);
    basic_string& insert(size_type pos1, const basic_string& str,
                         size_type pos2, size_type n);
    basic_string& insert(size_type pos, const charT* s, size_type n);
    basic_string& insert(size_type pos, const charT* s);
    basic_string& insert(size_type pos, size_type n, charT c);
    iterator insert(iterator p, charT c = charT());
    void     insert(iterator p, size_type n, charT c);
//    template<class InputIterator>
//      void insert(iterator p, InputIterator first, InputIterator last);
    void insert(iterator p, charT* first, charT* last);
    basic_string& erase(size_type pos = 0, size_type n = npos);
    iterator erase(iterator position);
    iterator erase(iterator first, iterator last);
    basic_string& replace(size_type pos1, size_type n1, const basic_string& str);
    basic_string& replace(size_type pos1, size_type n1, const basic_string& str,
                          size_type pos2, size_type n2);
    basic_string& replace(size_type pos, size_type n1, const charT* s,
                          size_type n2);
    basic_string& replace(size_type pos, size_type n1, const charT* s);
    basic_string& replace(size_type pos, size_type n1, size_type n2, charT c);
    basic_string& replace(iterator i1, iterator i2, const basic_string& str);
    basic_string& replace(iterator i1, iterator i2, const charT* s, size_type n);
    basic_string& replace(iterator i1, iterator i2, const charT* s);
    basic_string& replace(iterator i1, iterator i2, size_type n, charT c);
    //template<class InputIterator>
    //  basic_string& replace(iterator i1, iterator i2,
    //                        InputIterator j1, InputIterator j2);
    basic_string& replace(iterator i1, iterator i2, charT* j1, charT* j2);
    size_type copy(charT* s, size_type n, size_type pos = 0) const;
    void swap(basic_string<charT,traits,Allocator>&);
  // _lib.string.ops_ string operations:
    const charT* c_str() const;  // explicit
    const charT* data() const;
    const allocator_type& get_allocator() const;
    size_type find (const basic_string& str, size_type pos = 0) const;
    size_type find (const charT* s, size_type pos, size_type n) const;
    size_type find (const charT* s, size_type pos = 0) const;
    size_type find (charT c, size_type pos = 0) const;
    size_type rfind(const basic_string& str, size_type pos = npos) const;
    size_type rfind(const charT* s, size_type pos, size_type n) const;
    size_type rfind(const charT* s, size_type pos = npos) const;
    size_type rfind(charT c, size_type pos = npos) const;
    size_type find_first_of(const basic_string& str, size_type pos = 0) const;
    size_type find_first_of(const charT* s, size_type pos, size_type n) const;
    size_type find_first_of(const charT* s, size_type pos = 0) const;
    size_type find_first_of(charT c, size_type pos = 0) const;
    size_type find_last_of (const basic_string& str,
                            size_type pos = npos) const;
    size_type find_last_of (const charT* s, size_type pos, size_type n) const;
    size_type find_last_of (const charT* s, size_type pos = npos) const;
    size_type find_last_of (charT c, size_type pos = npos) const;

    size_type find_first_not_of(const basic_string& str,
                                size_type pos = 0) const;
    size_type find_first_not_of(const charT* s, size_type pos,
                                size_type n) const;
    size_type find_first_not_of(const charT* s, size_type pos = 0) const;
    size_type find_first_not_of(charT c, size_type pos = 0) const;
    size_type find_last_not_of (const basic_string& str,
                                size_type pos = npos) const;
    size_type find_last_not_of (const charT* s, size_type pos,
                                size_type n) const;
    size_type find_last_not_of (const charT* s, size_type pos = npos) const;
    size_type find_last_not_of (charT c, size_type pos = npos) const;
    basic_string substr(size_type pos = 0, size_type n = npos) const;
    int compare(const basic_string& str) const;
    int compare(size_type pos1, size_type n1,
                const basic_string& str) const;
    int compare(size_type pos1, size_type n1,
                const basic_string& str,
                size_type pos2, size_type n2) const;
    int compare(const charT* s) const;
    int compare(size_type pos1, size_type n1,
                const charT* s, size_type n2 = npos) const;
  };

typedef basic_string<char,string_char_traits_char,allocator_char> std_string;

class std_string2 {
  public:
  // types:
    //typedef          traits            traits_type;
    //typedef /*typename*/ traits::char_type value_type;
    typedef char value_type;
    typedef size_t size_type;
    typedef ptrdiff_t difference_type;
    typedef char& reference;
    typedef const char& const_reference;
    typedef char* pointer;
    typedef const char* const_pointer;
    //typedef          Allocator         allocator_type;
    //typedef /*typename*/ Allocator::size_type       size_type;
    //typedef /*typename*/ Allocator::difference_type difference_type;
    //typedef /*typename*/ Allocator::types_char::reference       reference;
    //typedef /*typename*/ Allocator::types_char::const_reference const_reference;
    //typedef /*typename*/ Allocator::types_char::pointer         pointer;
    //typedef /*typename*/ Allocator::types_char::const_pointer   const_pointer;
    //typedef implementation_defined                      iterator;
    //typedef implementation_defined                      const_iterator;
    //typedef reverse_iterator<iterator, value_type,
    //                reference, difference_type>               reverse_iterator;
    //typedef reverse_iterator<const_iterator, value_type,
    //                const_reference, difference_type>         const_reverse_iterator;
    typedef char*                      iterator;
    typedef const char*                const_iterator;
    typedef char*                      reverse_iterator;
    typedef const char*                const_reverse_iterator;
    static const size_type npos;// = -1;
    ////*explicit*/ basic_string(const Allocator& = Allocator());
    std_string2();
    std_string2(const std_string2& str, size_type pos = 0, size_type n = npos);
    std_string2(const char* s, size_type n);
    std_string2(const char* s);
    std_string2(size_type n, char c);
    std_string2(char*,char*);
   ~std_string2();
    std_string2& operator=(const std_string2& str);
    std_string2& operator=(const char* s);
    std_string2& operator=(char c);

    iterator       begin();
    const_iterator begin() const;
    iterator       end();
    const_iterator end() const;

    reverse_iterator       rbegin();
    const_reverse_iterator rbegin() const;
    reverse_iterator       rend();
    const_reverse_iterator rend() const;
  // _lib.string.capacity_ capacity:
    size_type size() const;
    size_type length() const;
    size_type max_size() const;
    void resize(size_type n, char c);
    void resize(size_type n);
    size_type capacity() const;
    void reserve(size_type res_arg);
    bool empty() const;
  // _lib.string.access_ element access:
    char      operator[](size_type pos) const;
    reference operator[](size_type pos);
    const_reference at(size_type n) const;
    reference       at(size_type n);
  // _lib.string.modifiers_ modifiers:
    std_string2& operator+=(const std_string2& rhs);
    std_string2& operator+=(const char* s);
    std_string2& operator+=(char c);
    std_string2& append(const std_string2& str);
    std_string2& append(const std_string2& str, size_type pos,
                         size_type n);
    std_string2& append(const char* s, size_type n);
    std_string2& append(const char* s);
    std_string2& append(size_type n, char c);
    //template<class InputIterator>
    //  basic_string& append(InputIterator first, InputIterator last);
    std_string2& append(char* first, char* last);
    std_string2& assign(const std_string2&);
    std_string2& assign(const std_string2& str, size_type pos, size_type n);
    std_string2& assign(const char* s, size_type n);
    std_string2& assign(const char* s);
    std_string2& assign(size_type n, char c);
    //template<class InputIterator>
    //  basic_string& assign(InputIterator first, InputIterator last);
    std_string2& assign(char* first, char* last);

    std_string2& insert(size_type pos1, const std_string2& str);
    std_string2& insert(size_type pos1, const std_string2& str,
                         size_type pos2, size_type n);
    std_string2& insert(size_type pos, const char* s, size_type n);
    std_string2& insert(size_type pos, const char* s);
    std_string2& insert(size_type pos, size_type n, char c);
    iterator insert(iterator p, char c = char());
    void     insert(iterator p, size_type n, char c);
//    template<class InputIterator>
//      void insert(iterator p, InputIterator first, InputIterator last);
    void insert(iterator p, char* first, char* last);
    std_string2& erase(size_type pos = 0, size_type n = npos);
    iterator erase(iterator position);
    iterator erase(iterator first, iterator last);
    std_string2& replace(size_type pos1, size_type n1, const std_string2& str);
    std_string2& replace(size_type pos1, size_type n1, const std_string2& str,
                          size_type pos2, size_type n2);
    std_string2& replace(size_type pos, size_type n1, const char* s,
                          size_type n2);
    std_string2& replace(size_type pos, size_type n1, const char* s);
    std_string2& replace(size_type pos, size_type n1, size_type n2, char c);
    std_string2& replace(iterator i1, iterator i2, const std_string2& str);
    std_string2& replace(iterator i1, iterator i2, const char* s, size_type n);
    std_string2& replace(iterator i1, iterator i2, const char* s);
    std_string2& replace(iterator i1, iterator i2, size_type n, char c);
    //template<class InputIterator>
    //  basic_string& replace(iterator i1, iterator i2,
    //                        InputIterator j1, InputIterator j2);
    std_string2& replace(iterator i1, iterator i2, char* j1, char* j2);
    size_type copy(char* s, size_type n, size_type pos = 0) const;
    void swap(std_string2&);
  // _lib.string.ops_ string operations:
    const char* c_str() const;  // explicit
    const char* data() const;
    const allocator_type& get_allocator() const;
    size_type find (const std_string2& str, size_type pos = 0) const;
    size_type find (const char* s, size_type pos, size_type n) const;
    size_type find (const char* s, size_type pos = 0) const;
    size_type find (char c, size_type pos = 0) const;
    size_type rfind(const std_string2& str, size_type pos = npos) const;
    size_type rfind(const char* s, size_type pos, size_type n) const;
    size_type rfind(const char* s, size_type pos = npos) const;
    size_type rfind(char c, size_type pos = npos) const;
    size_type find_first_of(const std_string2& str, size_type pos = 0) const;
    size_type find_first_of(const char* s, size_type pos, size_type n) const;
    size_type find_first_of(const char* s, size_type pos = 0) const;
    size_type find_first_of(char c, size_type pos = 0) const;
    size_type find_last_of (const std_string2& str,
                            size_type pos = npos) const;
    size_type find_last_of (const char* s, size_type pos, size_type n) const;
    size_type find_last_of (const char* s, size_type pos = npos) const;
    size_type find_last_of (char c, size_type pos = npos) const;

    size_type find_first_not_of(const std_string2& str,
                                size_type pos = 0) const;
    size_type find_first_not_of(const char* s, size_type pos,
                                size_type n) const;
    size_type find_first_not_of(const char* s, size_type pos = 0) const;
    size_type find_first_not_of(char c, size_type pos = 0) const;
    size_type find_last_not_of (const std_string2& str,
                                size_type pos = npos) const;
    size_type find_last_not_of (const char* s, size_type pos,
                                size_type n) const;
    size_type find_last_not_of (const char* s, size_type pos = npos) const;
    size_type find_last_not_of (char c, size_type pos = npos) const;
    std_string2 substr(size_type pos = 0, size_type n = npos) const;
    int compare(const std_string2& str) const;
    int compare(size_type pos1, size_type n1,
                const std_string2& str) const;
    int compare(size_type pos1, size_type n1,
                const std_string2& str,
                size_type pos2, size_type n2) const;
    int compare(const char* s) const;
    int compare(size_type pos1, size_type n1,
                const char* s, size_type n2 = npos) const;
  };

void foo()
{
  std_string a;
  a = "fred";
}